#!/usr/bin/env python

"A WebStack application for a system configurator."

import WebStack.Generic
from XSLForms.Resources.WebResources import \
    XSLFormsResource, input, output, resources, prepare_resources as xslforms_prepare_resources
import os

# Site map imports.

from WebStack.Resources.ResourceMap import MapResource
from WebStack.Resources.Selectors import EncodingSelector
from WebStack.Resources.Static import DirectoryResource

# Configuration settings.

encoding = "utf-8"

# Resource classes.

class ConfiguratorResource(XSLFormsResource):

    "A resource providing a system configurator."

    resource_dir = resources(__file__)
    template_resources = {
        "configuration" : output("config_template.xhtml")
        }
    init_resources = {
        "configuration" : input("config_template.xhtml")
        }
    transform_resources = {
        "filter" : ["config_filter.xsl"]
        }
    document_resources = {
        "accessories" : "config_accessories.xml",
        "base-system" : "config_base_system.xml",
        "cpu" : "config_cpu.xml",
        "hard-disk" : "config_hard_disk.xml",
        "keyboard" : "config_keyboard.xml",
        "memory-unit" : "config_memory_unit.xml",
        "mouse" : "config_mouse.xml",
        "screen" : "config_screen.xml",
        "storage-unit" : "config_storage_unit.xml",
        "translations" : "translations.xml"
        }
    in_page_resources = {
        "cpu" : ("configuration", "config_output_cpu.xsl", "cpu-node"),
        "memory" : ("configuration", "config_output_memory.xsl", "memory-node"),
        "hard-disks" : ("configuration", "config_output_harddisks.xsl", "hard-disks-node"),
        "accessories" : ("configuration", "config_output_accessories.xsl", "accessories-node")
        }

    def select_activity(self, trans, form):
        self.set_activity(trans, "configuration")

    def respond_to_input(self, trans, form):

        # Creating selected elements.

        selectors = form.get_selectors(create=1)

        # Add and remove elements according to the selectors found.

        self.add_elements(selectors.get("add-memory-unit"), "memory-unit")
        self.remove_elements(selectors.get("remove-memory-unit"))
        self.add_elements(selectors.get("add-storage-unit"), "storage-unit")
        self.remove_elements(selectors.get("remove-storage-unit"))
        self.add_elements(selectors.get("add-hard-disk"), "hard-disk")
        self.remove_elements(selectors.get("remove-hard-disk"))

        # Send a response according to certain parameters.
        # When exported, an XML version of the data is returned.

        parameters = form.get_parameters()

        if parameters.has_key("export"):
            trans.set_content_type(WebStack.Generic.ContentType("text/xml", encoding))
            self.get_document(trans).toStream(trans.get_response_stream(), trans.get_response_stream_encoding())
            raise WebStack.Generic.EndOfResponse

    def init_document(self, trans, form):

        # Transform, adding enumerations/ranges.

        XSLFormsResource.init_document(self, trans, form, 
            references={
                "accessories" : self.prepare_document("accessories"),
                "base-system" : self.prepare_document("base-system"),
                "cpu" : self.prepare_document("cpu"),
                "hard-disk" : self.prepare_document("hard-disk"),
                "keyboard" : self.prepare_document("keyboard"),
                "memory-unit" : self.prepare_document("memory-unit"),
                "mouse" : self.prepare_document("mouse"),
                "screen" : self.prepare_document("screen"),
                "storage-unit" : self.prepare_document("storage-unit")
            })

    def respond_to_document(self, trans, form):

        # Filter out inappropriate choices.

        filter_xsl_list = self.prepare_transform("filter")
        self.set_document(trans, self.get_result(filter_xsl_list, self.get_document(trans)))

    def create_output(self, trans, form):

        # Complete the response.

        stylesheet_parameters = {}

        try:
            stylesheet_parameters["locale"] = trans.get_content_languages()[0]
        except IndexError:
            pass

        XSLFormsResource.create_output(self, trans, form,
            stylesheet_parameters=stylesheet_parameters,
            references={
                "translations" : self.prepare_document("translations")
            })

# Site map initialisation.

def get_site():

    "Return a simple Web site resource."

    # Get the main resource and the directory used by the application.

    configurator_resource = ConfiguratorResource()
    directory = configurator_resource.resource_dir

    # Make a simple Web site.

    resource = MapResource({
        # Static resources:
        "styles" : DirectoryResource(os.path.join(directory, "styles"), {"css" : "text/css"}),
        "scripts" : DirectoryResource(os.path.join(directory, "scripts"), {"js" : "text/javascript"}),
        # Main page:
        "" : configurator_resource,
        # Fragments:
        "cpu" : configurator_resource,
        "memory" : configurator_resource,
        "hard-disks" : configurator_resource,
        "accessories" : configurator_resource
        })

    return EncodingSelector(resource, encoding)

# Resource preparation ahead of time - useful for making installations.

def prepare_resources():
    for cls in [ConfiguratorResource]:
        xslforms_prepare_resources(cls)

# vim: tabstop=4 expandtab shiftwidth=4
